#include <stdio.h>
#include <string.h>



int is_valid_display_num( const char *display_name )
{
    const char    delimiters[]        = ".";
          char    display_num_str[80] = " ";
          char   *cptr;

    if ( cptr = strrchr( display_name, ':' ) ) {
         strcpy( display_num_str, cptr+1 );
         if ( cptr = strtok( display_num_str, delimiters ) ) {
             if ( ! isdigit( cptr[0] ) ) {
                 fprintf( stderr, __FILE__":is_valid_display_num() - \n"
                                  "\t%s has invalid display number at 1st "
                                  "digit\n", display_name );
                 fflush( stderr );
                 return 0;
             }
             if ( cptr = strtok( NULL, delimiters ) )
                 if ( ! isdigit( cptr[0] ) ) {
                     fprintf( stderr, __FILE__":is_valid_display_num() - \n"
                                      "\t%s has invalid display number at 2nd "
                                      "digit\n", display_name );
                     fflush( stderr );
                     return 0;
                 }
         }
    }

    return 1;
}



/*@ 
   exe_xterm_gplot - render 2D surface data or 1D X-Y data by GNUplot 
                     in X-Windows.  This assumes gnuplot is installed
                     and can be found in the path as "gnuplot".

  Unmodified Input Variable :
. gplot_data_name - full pathanme of the file containing the gnuplot
                    readable data, an ASCII file, where data are arranged
                    in column form.  For 3D data, (x, y, z) coordinates are
                    arranged in 3 separated columns, a blank line is inserted
                    whenever z-coordiantes is changed.  For 2D data, no blank
                    line is needed.
. gplot_template_name - the filename of GNUplot template command file
                        where "gplot_data_name", the 1st argument of this
                        subroutine, will replace "DATAFILE-PATHNAME" in
                        gplot_template_name.  
. gplot_cmd_file - the filename of the actual gnuplot command file, 
                   It is the resulting file of the insertion of 
                   "gplot_data_name" in "gplot_template_name".
                   i.e. gplot_cmd_file = gplot_template_name( gplot_data_name )
. title - string of the title to be used in xterm's and gnuplot_x11's title-bar,
          If title == NULL, default title will be used
. display - name of the display/machine to be used.  If display == NULL or
            strlen( display ) == 0, then display option in xterm and gnuplot
            will NOT be set
. xterm_geom_coord - Integer array of 2, specify the location of the Xterm
                     in the unit of pixel.  If coord[ ii ] is negative,
                     NO xterm geometry coordinate will be set.
. gplot_geom_size  - Integer array of 2, specify the size of the GNUplot
                     in the unit of pixel.  If size[ ii ] is zero or negative
                     NO gplot geometry szie will be set.
. gplot_geom_coord - Integer array of 2, specify the location of the GNUplot
                     in the unit of pixel.  If coord[ ii ] is negative,
                     NO gplot geometry coordinate will be set.
.n


  Notes on the usage :
  The second argument of the subroutine, gplot_template_name, which is a
  template needed by this subroutine to successfully run gnuplot as
  the program runs.  Typical templates are included in the distribution 
  tree, $(RGX)/demo.  GNUPLOT_template_1D.cmd is for 1D data, a X-Y graph.  
  GNUPLOT_template_2D_mov.cmd and GNUPLOT_template_2D_pic.cmd are for 2D 
  surface data, a (X,Y)-Z graph.  Where "mov" stands for movie, an animated
  rotation of 2D surface data.  And "pic" is a static view of the 2D 
  surface data.   Example of usage of this routine can be found in
  $(RGX)/src/demo-laplace/exe_gplot_Task_Grid.c and laplace_main.c.
.n
.n This routine requires str_replace.c to compile.


.N RGX_fortran

  Additional Notes on the Fortran Interface :
  Since the subroutine is defined in C, and most of what subroutine
  does is string manupilation using string.h.  So user is assumed to
  put a NULL character, i.e. "\0", at the end of all the character 
  arrays variables for termination, namely first 5 variables in the
  argument list of this subroutine. 


.N RGX_not_included

@*/

int exe_xterm_gplot( 
                     char          *gplot_data_name,
                     char          *gplot_template_name,
                     char          *gplot_cmd_file,
                     char          *title,
                     char          *display,
                     int            xterm_geom_coord[ 2 ],
                     int            gplot_geom_size[ 2 ],
                     int            gplot_geom_coord[ 2 ] 
                   )
{
#  define    line_Max 80

   /*  Local Variables  */
   const char   SED_com[]                     = "sed -e ";
   char         SED_arg[ 2*line_Max ];
   const char   DATAFILE[]                    = "DATAFILE-PATHNAME";
   const char   XTERM_com[]                   = "xterm -fn 5x7 -geom 80x5";
   const char   GNUPLOT_com[]                 = "-e gnuplot ";
   const char   PLOTconsole[]                 = "-title PLOTconsole@";
   const char   GNUplot_hdr[]                 = "-title GNUplot@";
   const char   display_opt[]                 = "-display ";
   const char   geom_opt[]                    = "-geometry ";
   const char   geom_size_fmt[]               = "%04dx%04d";
   const char   geom_coord_fmt[]              = "+%04d+%04d";
   char         xterm_geom_coord_str[ line_Max ];
   char         gplot_geom_size_str[ line_Max ];
   char         gplot_geom_coord_str[ line_Max ];
   char         display_opt_str[ line_Max ];
   char         gplot_geom_opt_str[ line_Max ];

   /*  Local Dummy variables  */
   char         tmp_filename[ 2*line_Max ];
   char         local_cmd_str[ 4*line_Max ];
   int          ierr;
   
   
   /* "." in filename does NOT need to be replaced by "\\."  */
   /* 'Cause  "." is recognized in replacing string in SED,  */
   /* But "." is NOT recognized as normal text in replaced string in SED     */
   /* replace "/" by "\\/" in the input filename save it in a tmp_filename  */

   str_replace( "/", "\\/", gplot_data_name, tmp_filename );

   /* prepare for the SED's argument, SED_arg  */
   strcpy( SED_arg, "\'s/" );
   strcat( SED_arg, DATAFILE );
   strcat( SED_arg, "/" );
   strcat( SED_arg, tmp_filename );
   strcat( SED_arg, "/g\'" );

   /* Remove the gplot_cmd_file, if it exists */
   sprintf( local_cmd_str, "rm -rf " );
   strcat( local_cmd_str, gplot_cmd_file );
   ierr = system(  local_cmd_str );

   /* prepare the string to create the gplot_cmd_file  */
   strcpy( local_cmd_str, SED_com );
   strcat( local_cmd_str, SED_arg );
   strcat( local_cmd_str, " " );
   strcat( local_cmd_str, gplot_template_name );
   strcat( local_cmd_str, " > " );
   strcat( local_cmd_str, gplot_cmd_file );

   /* Create the gplot_cmd_file from the gplot_template  */
   ierr = system( local_cmd_str );
   /*
   fprintf( stdout, "%s \n\n", local_cmd_str ); 
   fflush( stdout );
   */

   /* Create XTERM and its argument to execute GNUplot of gplot_cmd_file */

   if ( xterm_geom_coord[0] >= 0 && xterm_geom_coord[1] >= 0 )
      sprintf( xterm_geom_coord_str, geom_coord_fmt,
               xterm_geom_coord[0], xterm_geom_coord[1] );
   else
      strcpy( xterm_geom_coord_str, "" );

   if ( gplot_geom_size[0] > 0 && gplot_geom_size[1] > 0 )
      sprintf( gplot_geom_size_str, geom_size_fmt,
               gplot_geom_size[0], gplot_geom_size[1] );
   else
      strcpy( gplot_geom_size_str, "" );

   if ( gplot_geom_coord[0] >= 0 && gplot_geom_coord[1] >= 0 )
      sprintf( gplot_geom_coord_str, geom_coord_fmt,
               gplot_geom_coord[0], gplot_geom_coord[1] );
   else
      strcpy( gplot_geom_coord_str, "" );

   if ( strlen( gplot_geom_size_str ) + strlen( gplot_geom_coord_str ) > 0 )
   {
      strcpy( gplot_geom_opt_str, geom_opt );
      strcat( gplot_geom_opt_str, gplot_geom_size_str );
      strcat( gplot_geom_opt_str, gplot_geom_coord_str );
   }
   else
      strcpy( gplot_geom_opt_str, "" );

   if ( strlen( display ) > 0 )
   {
      strcpy( display_opt_str, display_opt );
      strcat( display_opt_str, display );
      /*  
          if it contains Invalid display number, discard the invalid numbers 
          and assume the default
      */
      if ( ! is_valid_display_num( display ) ) 
          strcpy( strrchr( display_opt_str, ':' ), ":0.0" );
   }  
   else
      strcpy( display_opt_str, "" );

   strcpy( local_cmd_str, XTERM_com );
   strcat( local_cmd_str, xterm_geom_coord_str );
   strcat( local_cmd_str, " " );
   strcat( local_cmd_str, display_opt_str );
   strcat( local_cmd_str, " " );
   strcat( local_cmd_str, PLOTconsole );
   strcat( local_cmd_str, title );
   strcat( local_cmd_str, " " );
   strcat( local_cmd_str, GNUPLOT_com );
   strcat( local_cmd_str, display_opt_str );
   strcat( local_cmd_str, " " );
   strcat( local_cmd_str, gplot_geom_opt_str );
   strcat( local_cmd_str, " " );
   strcat( local_cmd_str, GNUplot_hdr );
   strcat( local_cmd_str, title );
   strcat( local_cmd_str, " " );
   strcat( local_cmd_str, gplot_cmd_file );
   strcat( local_cmd_str, " &" );

   /* EXcute GNUPLOT  */
   ierr = system( local_cmd_str ); 
   /*
   fprintf( stdout, "%s \n\n", local_cmd_str );
   fflush( stdout );
   */
}


#ifdef FortranNoUnderscore
#   define exe_xterm_gplot__  exe_xterm_gplot
#endif

#ifdef FortranSingleUnderscore
#   define exe_xterm_gplot__  exe_xterm_gplot_
#endif

void exe_xterm_gplot__(
                        char          *gplot_data_name,
                        char          *gplot_template_name,
                        char          *gplot_cmd_file,
                        char          *title,
                        char          *display,
                        int            xterm_geom_coord[ 2 ],
                        int            gplot_geom_size[ 2 ],
                        int            gplot_geom_coord[ 2 ],
                        int           *ierr
                      )
{
   *ierr = exe_xterm_gplot( gplot_data_name, gplot_template_name,
                            gplot_cmd_file, title, display,
                            xterm_geom_coord, gplot_geom_size,
                            gplot_geom_coord );
}
