#include <stdio.h>
#include "mpi.h"



/*  
     Broadcast to every task/node in given Communicator,Cart_Grids_Comm, about
       Global Grid Size in each dim:  Glb_Grid_sz[ 0..Ndim-1 ]
     & stride_width of Task Grid in each dim:  stride_width[ 0..Ndim-1 ]
*/

/*@

  RGX_Set_Glb_Grid_Ppty - Set the Global Grid's properties, i.e.
                          Global Grid size, Glb_Grid_sz[ Ndim ] and
                          stride width of the ghost-point region,
                          stride_width[ Ndim ].  Broadcast these 2 sets of
                          values to all the tasks in the global grid
                          communicator 

  Unmodified Input Variables :
. Cart_Grids_Comm - Input Communicator containing all processors for the grid
. Ndim - Dimensionality of the Grid (Integer)
. Glb_Grid_sz - Integer Array, of size Ndim, specifies the maximum grid size
                of the global grid.  Glb_Grid_sz[ ii ] is the maximum grid size
                in the dimension ii, where the minimum is assumed to be 1.
. stride_width - Integer Array, of size Ndim, of the stride width of the task
                 grid.  It is implicitly assumed to be declared like
                 stride_width[ Ndim ].

  Modified Output Variables :
. returned value - returns MPI_SUCCESS if completed successfully
.n

.N RGX_fortran
.N RGX_minimal_set_of_variables
.N RGX_supported_datatype
.N RGX_yes_included

@*/
int RGX_Set_Glb_Grid_Ppty(
                           MPI_Comm  Cart_Grids_Comm,
                           int       Ndim,
                           int       Glb_Grid_sz[],
                           int       stride_width[]
                         )
{
    /*  local Variables :  */
    int master_tid = 0;
    int ierr[2];


    ierr[0] = MPI_Bcast( Glb_Grid_sz,  Ndim, MPI_INT, master_tid,
                         Cart_Grids_Comm );
    ierr[1] = MPI_Bcast( stride_width, Ndim, MPI_INT, master_tid,
                         Cart_Grids_Comm );
    if ( ierr[0] != MPI_SUCCESS )
    {
       fprintf( stderr, __FILE__": MPI_Bast( Glb_Grid_sz[] .. ) fails "
                        "with ierr = %d\n", ierr[0] );
       return ierr[0];
    }
    if ( ierr[1] != MPI_SUCCESS )
    {
       fprintf( stderr, __FILE__": MPI_Bast( stride_width[] .. ) fails "
                        "with ierr = %d\n", ierr[1] );
       return ierr[1];
    }

    return MPI_SUCCESS;
}



#ifdef FortranNoUnderscore
#  define rgx_set_glb_grid_ppty__  rgx_set_glb_grid_ppty
#endif

#ifdef FortranSingleUnderscore
#  define rgx_set_glb_grid_ppty__  rgx_set_glb_grid_ppty_
#endif

/*  Fortran Interface for RGX_Set_Glb_Grid_Ppty()  */
void rgx_set_glb_grid_ppty__(
                              MPI_Comm  *Cart_Grids_Comm,
                              int       *Ndim,
                              int       *Glb_Grid_sz,
                              int       *stride_width,
                              int       *ierr
                            )
{
   *ierr = RGX_Set_Glb_Grid_Ppty( *Cart_Grids_Comm, *Ndim,
                                  Glb_Grid_sz, stride_width );
}
